#include "projects/project.hpp"
#include "utils/local.hpp"

#include <filesystem>

namespace fs = std::filesystem;

Project::Project(const std::string& path, const std::string& path_to_scripts,
                 const std::string& name)
    : path_(path), path_to_scripts_(path_to_scripts), name_(name) {}

const fs::path& Project::GetPath() const {
  return path_;
}

const fs::path& Project::GetPathToScripts() const {
  return path_to_scripts_;
}

const std::string& Project::GetName() const {
  return name_;
}

bool Project::IsSubDirectory(const fs::path& path) const {
  return path.string().starts_with(path_.string());
}

void Project::SetPath(const std::string& path) {
  path_ = path;
}

void Project::SetPathToScripts(const std::string& path_to_scripts) {
  path_to_scripts_ = path_to_scripts;
}

void Project::SetName(const std::string& name) {
  name_ = name;
}

void Project::PrintInfo() const {
  std::cout << "Project Name: " << name_ << "\n";
  std::cout << "Path: " << path_ << "\n";
  std::cout << "Path to Scripts: " << path_to_scripts_ << "\n";
}

Project CreateProject() {
  fs::path current_path = fs::current_path();
  return CreateProject(current_path.filename().string());
}

Project CreateProject(const std::string& name) {
  fs::path current_path = fs::current_path();
  std::string path = current_path.string();

  std::string path_to_scripts = GetLocalPath() / "projects" / name;
  fs::create_directories(path_to_scripts);

  return Project(path, path_to_scripts, name);
}
